@@ -8,9 +8,9 @@ import warnings
 from collections import OrderedDict
 from contextlib import redirect_stdout, ExitStack
 from datetime import datetime
-from .outputs import OutputSaver, OutputRequest
+from slippy.core.outputs import OutputSaver, OutputRequest
 
-from slippy.abcs import _SurfaceABC, _LubricantModelABC, _ContactModelABC
+from slippy.core import _SurfaceABC, _LubricantModelABC, _ContactModelABC
 from slippy.contact.steps import _ModelStep, InitialStep
 
 __all__ = ['ContactModel']
@@ -46,10 +46,8 @@ class ContactModel(_ContactModelABC):
     -------
     add_step
         Adds a step object to the model
-    add_field_output
-        Adds a field output to the model
-    add_history_output
-        Adds a history output to the model
+    add_output
+        Adds an output request to the model or selected steps
     data_check
         Performs analysis checks for each of the steps and the model as a whole, prints the results to the log file
     solve
@@ -73,6 +71,8 @@ class ContactModel(_ContactModelABC):
         if lubricant is not None:
             self.lubricant_model = lubricant
         self.steps = OrderedDict({'Initial': InitialStep()})
+        self.current_step = None
+        self.current_step_start_time = None
         if output_dir is not None:
             if not os.path.isdir(output_dir):
                 try:
@@ -201,11 +201,15 @@ class ContactModel(_ContactModelABC):
         current_state: dict
             A dictionary containing the final state of the model
 
+        Notes
+        -----
+        Most steps produce detailed logging information that can be found in the log file.
+
         """
         if os.path.exists(self.log_file_name):
             os.remove(self.log_file_name)
 
-        current_state = None
+        current_state = {'time': 0.0}
 
         with ExitStack() as stack:
             output_writer = stack.enter_context(OutputSaver(self.name))
@@ -220,6 +224,8 @@ class ContactModel(_ContactModelABC):
 
             for this_step in self.steps:
                 print(f"Solving step {this_step}")
+                self.current_step = self.steps[this_step]
+                self.current_step_start_time = current_state['time']
                 for output in self.steps[this_step].outputs:
                     output.new_step(current_state['time'])
                 current_state = self.steps[this_step].solve(current_state, output_writer)
